""" Helper functions. """

# COPYRIGHT:   (c) Awasu Pty. Ltd. 2015-2020 (all rights reserved).
#              Unauthorized use of this code is prohibited.
#
# LICENSE:     This software is provided 'as-is', without any express
#              or implied warranty.
#
#              In no event will the author be held liable for any damages
#              arising from the use of this software.
#
#              Permission is granted to anyone to use this software
#              for any purpose, and to alter it and redistribute it freely,
#              subject to the following restrictions:
#
#              - The origin of this software must not be misrepresented;
#                you must not claim that you wrote the original software.
#                If you use this software, an acknowledgement is requested
#                but not required.
#
#              - Altered source versions must be plainly marked as such,
#                and must not be misrepresented as being the original software.
#                Altered source is encouraged to be submitted back to
#                the original author so it can be shared with the community.
#                Please share your changes.
#
#              - This notice may not be removed or altered from any
#                source distribution.

import sys
import re
import time
import datetime
import calendar
import xml.dom.minidom
import xml.parsers.expat
import unittest

# ---------------------------------------------------------------------

def safe_xml( val ):
    """Convert a value into something that's safe to insert into XML."""
    if val is None:
        return ""
    if type(val) is bytes:
        val = val.encode( "utf-8" )
    else:
        val = str( val )
    val = val.replace( "&", "&amp;" ).replace( "<", "&lt;" ).replace( ">", "&gt;" )
    val = val.replace( "\"", "&quot;" ) # nb: in case the value is an attribute
    return val

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

def pretty_xml( val ):
    """Prettify XML."""
    try:
        # NOTE: We remove blank lines generated by toprettyxml() (because it preserves
        # whitespace between XML tags).
        buf = xml.dom.minidom.parseString( val ).toprettyxml( indent="  " )
        lines = buf.split( "\n" )
        if lines[0] == "<?xml version=\"1.0\" ?>":
            del lines[0]
        return "\n".join( [ x for x in lines if x.strip() ] )
    except xml.parsers.expat.ExpatError:
        # NOTE: If the XML is invalid, we just return it verbatim.
        return "*** WARNING: Invalid XML ***\n" + val

# ---------------------------------------------------------------------

def parse_rfc2822_timestamp( tstamp ):
    """Parse an RFC 2822 timestamp."""

    # we sometimes get timestamps without the DOW :shrug:
    if re.match( "[A-Za-z]{3}, ", tstamp ):
        tstamp = tstamp[5:]

    # parse the date
    mo = re.match( "[0-9][0-9]?", tstamp ) # nb: we sometimes get single-digit dates :-/
    if not mo:
        return None # nb: we sometimes get weird rubbish :-/
    date = int( mo.group() )
    tstamp = tstamp[mo.end()+1:]

    # parse the rest of the timestamp
    #   0----5----0----5----0----5
    #   Apr 2014 15:07:51 +0000
    # NOTE: We can't use strptime(), since it will look for month names
    # using the current locale, but we need English.
    month_names = {
        "jan": 1, "feb": 2, "mar": 3, "apr": 4, "may": 5, "jun": 6,
        "jul": 7, "aug": 8, "sep": 9, "oct": 10, "nov": 11, "dec": 12
    }
    month = month_names[ tstamp[:3].lower() ]
    year = int( tstamp[4:8] )
    hours = int( tstamp[9:11] )
    minutes = int( tstamp[12:14] )
    seconds = int( tstamp[15:17] )
    tstamp2 = datetime.datetime( year, month, date, hours, minutes, seconds )

    # adjust for the time zone
    if tstamp[18] in ("+","-"):
        tz_delta = 60 * int(tstamp[19:21]) + int(tstamp[21:23])
        tz_delta *= -1 if tstamp[18] == "+" else +1
        tstamp2 += datetime.timedelta( seconds=60*tz_delta )

    return calendar.timegm( tstamp2.utctimetuple() )

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

def make_iso8601_timestamp( tstamp ):
    """Return an ISO 8601 timestamp (for Atom feeds)."""
    if type(tstamp) is int:
        tstamp = time.gmtime( tstamp )
    return "{year:4d}-{month:02d}-{date:02d}T{hours:02d}:{minutes:02d}:{seconds:02d}Z".format(
        year = tstamp.tm_year,
        month = tstamp.tm_mon,
        date = tstamp.tm_mday,
        hours = tstamp.tm_hour,
        minutes = tstamp.tm_min,
        seconds = tstamp.tm_sec
    )

# ---------------------------------------------------------------------

def make_atom_text_type( mime_type ):
    """Convert a MIME type to an Atom text type."""
    if mime_type == "text/plain":
        return "text"
    elif mime_type == "text/html":
        return "html"
    return "???"

# ---------------------------------------------------------------------

def dump_bytes( buf, caption="", prefix="", out=sys.stdout ):
    """Formatted byte dump."""
    if caption:
        print( caption, file=out ) # nb: no line prefix
    # dump the value as bytes
    for line_no in range( 0, int((len(buf)+15)/16) ):
        print( "{}{:04x}: ".format( prefix, 16*line_no ), end="", file=out )
        row_bytes = buf[ 16*line_no : 16*line_no+16 ]
        line_buf = " ".join( [ "{:02x}".format(ord(ch)) for ch in row_bytes ] )
        if len(line_buf) > 23: # nb: 23 = 8 hex values (2 digits each), plus 7 separators
            line_buf = "{} {}".format( line_buf[:23], line_buf[23:] )
        line_buf += " "*(48-len(line_buf))
        print( line_buf, end="", file=out )
        print(
            " | " + "".join( [ ch if 32 <= ord(ch) < 127 else "." for ch in row_bytes ] ),
            file=out
        )

# ---------------------------------------------------------------------

class UtilsTestCase( unittest.TestCase ):

    def test_safe_xml( self ):
        self.assertEqual( safe_xml('foo="<bar>"'), "foo=&quot;&lt;bar&gt;&quot;" )

    def test_rfc2822_timestamps( self ):
        self.assertEqual( parse_rfc2822_timestamp( "Tue, 01 Apr 2014 12:02:03 +0400" ),
            1396339323
        )
        self.assertEqual( parse_rfc2822_timestamp( "1 Apr 2014 12:02:03 +0400" ),
            1396339323
        )
        self.assertEqual( parse_rfc2822_timestamp( "foo" ),
            None
        )

    def test_iso8601_timestamp( self ):
        self.assertEqual(
            make_iso8601_timestamp( time.struct_time( ( 2001, 2, 3, 4, 5, 6, 0,1,0 ) ) ),
            "2001-02-03T04:05:06Z"
        )

# ---------------------------------------------------------------------

if __name__ == "__main__":
    # run the unit tests
    unittest.main()



""" Logging services. """

# COPYRIGHT:   (c) Awasu Pty. Ltd. 2015-2020 (all rights reserved).
#              Unauthorized use of this code is prohibited.
#
# LICENSE:     This software is provided 'as-is', without any express
#              or implied warranty.
#
#              In no event will the author be held liable for any damages
#              arising from the use of this software.
#
#              Permission is granted to anyone to use this software
#              for any purpose, and to alter it and redistribute it freely,
#              subject to the following restrictions:
#
#              - The origin of this software must not be misrepresented;
#                you must not claim that you wrote the original software.
#                If you use this software, an acknowledgement is requested
#                but not required.
#
#              - Altered source versions must be plainly marked as such,
#                and must not be misrepresented as being the original software.
#                Altered source is encouraged to be submitted back to
#                the original author so it can be shared with the community.
#                Please share your changes.
#
#              - This notice may not be removed or altered from any
#                source distribution.

import time

_log_file = None

# ---------------------------------------------------------------------

def log_msg( msg, *args, **kwargs ):
    """Log a message."""
    if not _log_file:
        return
    if msg == "":
        _log_file.write( "\n" )
        return
    # log the message
    _log_file.write( "{} | ".format( time.strftime("%Y-%m-%d %H:%M:%S") ) )
    _log_file.write( msg.format( *args, **kwargs ) )
    _log_file.write( "\n" )
    _log_file.flush()

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

def log_raw_msg( msg ):
    """Log a raw message."""
    if not _log_file:
        return
    _log_file.write( msg )
    if not msg.endswith( "\n" ):
        _log_file.write( "\n" )
    _log_file.flush()

# ---------------------------------------------------------------------

def init_logging( log_filename ):
    """Initialize logging."""
    if not log_filename:
        return
    global _log_file
    if log_filename.startswith( "+" ):
        # append to the log file
        _log_file = open( log_filename[1:], "a", encoding="utf-8" )
        _log_file.write( "\n\n\n=== NEW SESSION ===\n" )
    else:
        # start a new log file
        _log_file = open( log_filename, "w", encoding="utf-8" )



""" Classes for generating feeds. """

# COPYRIGHT:   (c) Awasu Pty. Ltd. 2015-2020 (all rights reserved).
#              Unauthorized use of this code is prohibited.
#
# LICENSE:     This software is provided 'as-is', without any express
#              or implied warranty.
#
#              In no event will the author be held liable for any damages
#              arising from the use of this software.
#
#              Permission is granted to anyone to use this software
#              for any purpose, and to alter it and redistribute it freely,
#              subject to the following restrictions:
#
#              - The origin of this software must not be misrepresented;
#                you must not claim that you wrote the original software.
#                If you use this software, an acknowledgement is requested
#                but not required.
#
#              - Altered source versions must be plainly marked as such,
#                and must not be misrepresented as being the original software.
#                Altered source is encouraged to be submitted back to
#                the original author so it can be shared with the community.
#                Please share your changes.
#
#              - This notice may not be removed or altered from any
#                source distribution.

import sys
import os
import time

from awasu_tools.utils import safe_xml, pretty_xml
from awasu_tools.log import log_msg, log_raw_msg

# ---------------------------------------------------------------------

class Feed:
    """Container for a feed and its items."""

    def __init__( self, title, home_url, description=None, image_url=None, updated_time=None, extra_args=None ):
        """Initialize the Feed."""
        self.title = title
        self.home_url = home_url
        self.description = description
        self.image_url = image_url
        self.updated_time = updated_time if updated_time else time.time()
        self.extra_args = extra_args
        self.feed_items = []

    def get_xml( self, templ=None, log=None ):
        """Generate the feed XML."""
        # initialize
        if templ:
            assert type(templ) is str
        else:
            # use the default template
            templ = """<?xml version="1.0" encoding="UTF-8"?>""" "\n" \
                    """<feed xmlns="http://www.w3.org/2005/Atom" xmlns:xh="http://www.w3.org/1999/xhtml">""" "\n" \
                    """<title type="text">{title}</title>""" "\n" \
                    """<subtitle type="html">{description}</subtitle>""" "\n" \
                    """<link href="{url}" />""" "\n" \
                    """<logo>{image_url}</logo>""" "\n" \
                    """<updated>{updated_time}</updated>""" "\n" \
                    """{feed_items}""" "\n" \
                    """</feed>"""
        # generate the feed XML
        args = {
            "title": self.title,
            "description": self.description,
            "url": self.home_url,
            "image_url": self.image_url,
            "updated_time": _format_time( self.updated_time ),
        }
        if self.image_url and not self.image_url.startswith(("http://","https://","file://")):
            # NOTE: We assume image_url has been set to point to a file in our directory.
            ### IMPORTANT CAVEAT ###
            # If the channel image is accessed as a local file, it won't show properly
            # because of CORS (although it seems to work if the channel is opened in Awasu) :-/
            # Things will also definitely not work if the channel summary page is loaded
            # from another computer, via a remote Awasu API call, since the other computer
            # obviously won't have access to file on the local computer. We can live with this
            # for now; if it's really a problem for someone, they can copy the logo file
            # somewhere accessible, and override the image URL in the feed template.
            # FUDGE! The directory name could contain non-ASCII characters!
            dname = os.path.split( sys.argv[0] )[0].decode( "windows-1252" )
            args["image_url"] = "file:///" + os.path.join( dname, self.image_url )
        if self.extra_args:
            args.update( self.extra_args )
        args = { k: safe_xml(v) if v is not None else "" for k,v in args.items() }
        args["feed_items"] = "\n".join( [ fi.get_xml(log) for fi in self.feed_items ] )
        if not self.feed_items:
            # tidy up the output if there are no feed items
            pos = templ.find( "{feed_items}\n" )
            if pos >= 0:
                templ = templ[:pos] + templ[pos+13:]
        return templ.format( **args )

# ---------------------------------------------------------------------

class FeedItem:
    """Container for a feed item."""

    def __init__( self, title, url, content=None, updated_time=None, templ=None, extra_args=None ):
        """Initialize the FeedItem."""
        self.title = title
        self.url = url
        self.content = content
        self.updated_time = updated_time
        self.templ = templ
        self.extra_args = extra_args

    def get_xml( self, log=None ):
        """Generate the feed item XML."""
        # initialize
        if self.templ:
            assert type(self.templ) is str
            templ = self.templ
        else:
            # use the default template
            templ = """<entry>""" \
                    """<title type="text">{title}</title>""" \
                    """<link href="{url}" />""" \
                    """<updated>{updated_time}</updated>""" \
                    """<content type="html">{content}</content>""" \
                    """</entry>"""
        # prepare to generate the feed item XML
        args = {
            "title": self.title,
            "url": self.url,
            "updated_time": _format_time( self.updated_time ),
            "content": self.content,
        }
        if self.extra_args:
            args.update( self.extra_args )
        if log:
            log_msg( "" )
            log_msg( "Generating feed item..." )
            for key,val in args.items():
                log_msg( "- {} = {}", key, "" if val is None else val )
        # generate the feed item XML
        args = { k: safe_xml(v) if v is not None else "" for k,v in args.items() }
        buf = templ.format( **args )
        if log:
            log_msg( "Generated feed item XML:" )
            log_raw_msg( pretty_xml( buf ) )
        return buf

# ---------------------------------------------------------------------

def _format_time( val ):
    """Format a time value for insertion into a feed."""
    if type(val) is float:
        return time.strftime( "%Y-%m-%dT%H:%M:%SZ", time.gmtime(val) )
    else:
        return val

# ---------------------------------------------------------------------

if __name__ == "__main__":
    # a simple example
    feed = Feed( "An example feed", "http://test.com" )
    feed.feed_items.append(
        FeedItem( "Feed item 1", "http://test.com/item1", "This is feed item #1" )
    )
    feed.feed_items.append(
        FeedItem( "Feed item 2", "http://test.com/item2", "This is feed item #2", time.time() )
    )
    print( feed.get_xml() )



from distutils.core import setup

setup(
    name = "awasu-tools" ,
    version = "1.1" ,
    description = "Tools for writing Awasu extensions." ,
    long_description = open("README.rst","r").read() ,
    url = "https://awasu.com/awasu_tools" ,
    author = "Awasu" ,
    author_email = "support@awasu.com" ,
    packages = ["awasu_tools"]
)



""" Provide access to an Awasu config file. """

# COPYRIGHT:   (c) Awasu Pty. Ltd. 2015-2020 (all rights reserved).
#              Unauthorized use of this code is prohibited.
#
# LICENSE:     This software is provided 'as-is', without any express
#              or implied warranty.
#
#              In no event will the author be held liable for any damages
#              arising from the use of this software.
#
#              Permission is granted to anyone to use this software
#              for any purpose, and to alter it and redistribute it freely,
#              subject to the following restrictions:
#
#              - The origin of this software must not be misrepresented;
#                you must not claim that you wrote the original software.
#                If you use this software, an acknowledgement is requested
#                but not required.
#
#              - Altered source versions must be plainly marked as such,
#                and must not be misrepresented as being the original software.
#                Altered source is encouraged to be submitted back to
#                the original author so it can be shared with the community.
#                Please share your changes.
#
#              - This notice may not be removed or altered from any
#                source distribution.

import sys
import os
import io
import re
import unittest

# ---------------------------------------------------------------------

class ConfigFile:
    """Provide access to an Awasu config file."""

    # TextVal types
    # nb: see https://awasu.com/forums/viewtopic.php?p=32904#p32904
    TVT_UNKNOWN = 0 # nb: Awasu doesn't know if the value is plain-text or HTML
    TVT_PLAINTEXT = 1
    TVT_HTML = 2

    def __init__( self, src ):
        """Load an Awasu config file."""
        # load the config file
        if os.path.isfile( src ):
            with open( src, "r", encoding="utf-8" ) as fp:
                buf = fp.read()
            self.filename = src
        elif type(src) is bytes:
            buf = src.decode( "utf-8" )
            self.filename = None
        else:
            buf = str( src )
            self.filename = None
        if buf[0] == "\ufeff":
            buf = buf[1:] # nb: remove the BOM
        # NOTE: We can't use win32api.GetPrivateProfileXXX() since it barfs
        # on very long strings (~2K), so we parse the INI file manually :-/
        # However, we don't use ConfigParser, since we need full control
        # over how the file gets parsed.
        self.sections = {}
        curr_section_name = None
        for line_buf in io.StringIO( buf ):
            line_buf = line_buf.strip()
            if not line_buf or line_buf.startswith( ( "#", ";", "'" ) ):
                continue # nb: ignore comments and blank lines
            mo = re.search( r"^\[\s*([^]]+)\s*\]", line_buf )
            if mo:
                # found the start of a new section
                curr_section_name = mo.group(1).strip().lower()
                self.sections[ curr_section_name ] = {}
                continue
            mo = re.search( r"([^[=]+)\s*=", line_buf )
            if mo and curr_section_name:
                # found a new key/value pair
                key = mo.group(1).strip().lower()
                val = line_buf[ mo.end(0) : ].strip()
                self.sections[ curr_section_name ][ key ] = val

    def get_string( self, section, key, default="" ):
        """Return a string config value."""
        val, text_type = self.__get_val( section, key, default )
        assert text_type is None
        return val

    def get_textval( self, section, key, default=None ):
        """Return a string config value.

        Awasu uses TextVal's to store feed content (e.g. feed titles, or item descriptions),
        where it is important to know whether the content is plain-text or HTML.
        """
        if default is None:
            default = ( "", ConfigFile.TVT_UNKNOWN )
        else:
            assert len(default) == 2
            assert type( default[0] ) is str
            assert type( default[1] ) is int
        val, text_type = self.__get_val( section, key, default[0] )
        if text_type is None:
            text_type = default[1]
        assert text_type is not None
        return ( val, text_type )

    def get_string_list( self, section, default=None ):
        """Return a list of string values."""
        vals = []
        for i in range(1,sys.maxsize):
            val, text_type = self.__get_val( section, str(i), "" )
            assert text_type is None
            if not val:
                break
            vals.append( val )
        if vals:
            return vals
        else:
            return default if default is not None else []

    def get_string_indirect( self, section, key, default="", required=False ):
        """Return a string config value, possibly from a file.

        NOTE: Awasu doesn't use this feature, but it's useful when writing extensions.
        """
        val, text_type = self.__get_val( section, key, default )
        assert text_type is None
        if os.path.isfile( val ):
            # a file was specified - return the string from that
            with open( val, "r", encoding="utf-8" ) as fp:
                val = fp.read()
        else:
            if required:
                raise RuntimeError( "Can't find file: "+val )
        return val

    def get_int( self, section, key, default=0 ):
        """Return an integer config value."""
        val, text_type = self.__get_val( section, key, default )
        return int( val )

    def get_bool( self, section, key, default=False ):
        """Return a boolean config value."""
        val, text_type = self.__get_val( section, key, default )
        if val.lower() in ["1","true","yes","on","enable","enabled"]:
            return True
        if val.lower() in ["0","false","no","off","disable","disabled"]:
            return False
        raise ValueError( "Invalid boolean: "+val )

    def __get_val( self, section, key, default ):
        """Return a raw config value.

        NOTE: A value that is present, but empty, is treated the same as
        a value that is not present i.e. it will return the default value.
        """
        val = self.sections.get( section.lower(), {} ).get( key.lower() )
        if not val:
            # NOTE: We can't pass the default value into get(), since we need to handle both
            # the case where the key is not present, and where the key is present but empty.
            val = "" if default is None else str(default)
        # decode any %XX characters
        val = re.sub( "%[0-9A-Fa-f]{2}",
            lambda mo: chr( int( mo.group()[1:], 16 ) ),
            val
        )
        # check for an Awasu TextVal type flag
        mo = re.search( r"\n\*(\d)$", val )
        if mo:
            return ( val[:mo.start(0)], int(mo.group(1)) )
        else:
            return ( val, None )

    def dump( self, out=sys.stdout ):
        """Dump the ConfigFile."""
        for section,keyvals in self.sections.items():
            print( "[{}]".format(section), file=out )
            for key,val in keyvals.items():
                print( "{} = {}".format( key, val ), file=out )
            print( file=out )

# ---------------------------------------------------------------------

class ConfigFileTestCase( unittest.TestCase ):

    def test_get_val( self ):

        # initialize
        config_file = ConfigFile( b"""
[section 1]
foo=bar
  theAnswer  =  42
empty =

  [  SECTION 2  ] ; a comment
    FOO  =  !!!
#comment=foo
 ; comment = bar

[utf-8]
japan=\xE6\x97\xA5\xE6\x9C\xAC

[TextVal]
UnknownContent = This is some unknown content.%0A*0
PlainTextContent = This is some plain-text content.%0A*1
HtmlContent = This is some HTML content.%0A*2
BadTextValType = This content has a bad TextVal type.%0A*9
Empty=%0A*0
""" )

        # do simple tests
        self.assertEqual( config_file.get_string( "section 1", "foo" ), "bar" )
        self.assertEqual( config_file.get_int( "section 1", "theanswer" ), 42 )
        self.assertEqual( config_file.get_string( "section 2", "foo" ), "!!!" )
        self.assertEqual( config_file.get_string( "section 2", "theanswer" ), "" )
        self.assertEqual( config_file.get_string( "section 99", "foo" ), "" )

        # check that non-ASCII is being handled correctly
        val = config_file.get_string( "utf-8", "japan" )
        self.assertEqual( len(val), 2 )
        self.assertEqual( ord(val[0]), 0x65E5 )
        self.assertEqual( ord(val[1]), 0x672C )

        # check TextVal's
        self.assertEqual(
            config_file.get_textval( "TextVal", "UnknownContent" ),
            ( "This is some unknown content.", ConfigFile.TVT_UNKNOWN )
        )
        self.assertEqual(
            config_file.get_textval( "TextVal", "PlainTextContent" ),
            ( "This is some plain-text content.", ConfigFile.TVT_PLAINTEXT )
        )
        self.assertEqual(
            config_file.get_textval( "TextVal", "HtmlContent" ),
            ( "This is some HTML content.", ConfigFile.TVT_HTML )
        )
        self.assertEqual(
            config_file.get_textval( "TextVal", "BadTextValType" ),
            ( "This content has a bad TextVal type.", 9 )
        )
        self.assertEqual(
            config_file.get_textval( "TextVal", "Empty" ),
            ( "", ConfigFile.TVT_UNKNOWN )
        )

        # check handling of default values
        self.assertEqual(
            config_file.get_string( "section 1", "empty" ),
            ""
        )
        self.assertEqual(
            config_file.get_string( "section 1", "empty", "<default>" ),
            "<default>"
        )
        self.assertEqual(
            config_file.get_string( "section 1", "_not_present_", "<default>" ),
            "<default>"
        )
        self.assertEqual(
            config_file.get_string( "section 1", "_not_present_" ),
            ""
        )
        self.assertEqual(
            config_file.get_textval( "TextVal", "Empty" ),
            ( "", ConfigFile.TVT_UNKNOWN )
        )
        self.assertEqual(
            config_file.get_textval( "TextVal", "Empty", ("<default>",ConfigFile.TVT_HTML) ),
            ( "", ConfigFile.TVT_UNKNOWN ) # nb: because this key/value is present, but empty
        )
        self.assertEqual(
            config_file.get_textval( "TextVal", "_not_present_" ),
            ( "", ConfigFile.TVT_UNKNOWN )
        )
        self.assertEqual(
            config_file.get_textval( "TextVal", "_not_present_", ("<default>",ConfigFile.TVT_HTML) ),
            ( "<default>", ConfigFile.TVT_HTML )
        )

    def test_unicode( self ):

        # initialize
        config_file = ConfigFile( b"""
[section-\xE6\x97\xA5\xE6\x9C\xAC]
japan = \xE6\x97\xA5\xE6\x9C\xAC
\xE6\x97\xA5\xE6\x9C\xAC = nihon
""" )

        # check that non-ASCII is being handled correctly
        def check( key, expected_val ):
            val = config_file.get_string( "section-\u65e5\u672c", key )
            self.assertIs( type(val), str )
            self.assertEqual( val, expected_val )
        check( "japan", "\u65e5\u672c" )
        check( "\u65e5\u672c", "nihon" )

# ---------------------------------------------------------------------

if __name__ == "__main__":
    if len(sys.argv) == 1:
        # run the unit tests
        unittest.main()
    else:
        # load and dump the specified config file
        config_file = ConfigFile( sys.argv[1] )
        config_file.dump()
